use crate::ruleset::Card;
use xxhash_rust::xxh3::xxh3_64;

const ZOBRIST_CONSTANT: u32 = 0xba7f00d5;

pub struct Zobrist {
    pub value: u64
}

impl Zobrist {
    pub fn new() -> Zobrist {
        Zobrist { value: 0 }
    }
    pub fn toggle(&mut self, feature: Feature) {
        self.value ^= feature.zobrist()
    }
}


#[derive(Clone, Copy, Debug)]
pub enum Feature {
    CardAt { card: Card, slot: u8, depth: u8 },
    WellHas { card: Card, well: u8 }
}

impl Feature {
    pub fn zobrist(&self) -> u64 {
        let mut msg: [u8; 8] = [0; 8];
        msg[0..4].copy_from_slice(&ZOBRIST_CONSTANT.to_le_bytes());
        match *self {
            Self::CardAt { card, slot, depth } => msg[4..8].copy_from_slice(&[0, card.0, slot, depth]),
            Self::WellHas { card, well } => msg[4..7].copy_from_slice(&[1, card.0, well]),
        }
        xxh3_64(&msg)
    }
}